package app;

import generic.BaseObject;

import java.io.Serializable;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * @author Pfister
 */
public class Order extends BaseObject implements Serializable {

	private static final long serialVersionUID = -4176967988091913835L;

	private Integer id;
	
	private Customer customer;

	private Date date;

	private double priceTotal;

	/**
	 * @uml.property name="orderItems"
	 * @uml.associationEnd multiplicity="(0 -1)" inverse="order:app.OrderItem"
	 */
	private Set orderItems = new HashSet();

	/**
	 * Setter of the property <tt>orderItems</tt>
	 * 
	 * @param orderItemss
	 *            The orderItems to set.
	 * @uml.property name="orderItems"
	 */
	public void setOrderItems(Set set) {
		orderItems = set;
	}

	/**
	 * Getter of the property <tt>orderItems</tt>
	 * 
	 * @return Returns the orderItems.
	 * @uml.property name="orderItems"
	 */
	public Set getOrderItems() {
		return orderItems;
	}

	public Order() {
		this.date = new Date();
	}



	public void addOrderItem(OrderItem orderItem) {
		orderItem.setOrder(this);
		this.priceTotal = this.priceTotal + orderItem.getProduct().getPrice()
				* orderItem.getAmount();
		this.orderItems.add(orderItem);
	}

	public void removeOrderItem(OrderItem orderItem) {
		orderItem.setOrder(null);
		orderItems.remove(orderItem);
	}

	/**
	 * @return Returns the date.
	 * @uml.property name="date"
	 */
	public Date getDate() {
		return date;
	}

	/**
	 * @return Returns the id.
	 * @uml.property name="id"
	 */
	public Integer getId() {
		return id;
	}

	/**
	 * @return Returns the priceTotal.
	 * @uml.property name="priceTotal"
	 */
	public double getPriceTotal() {
		return priceTotal;
	}

	/**
	 * @param date
	 *            The date to set.
	 * @uml.property name="date"
	 */
	public void setDate(Date date) {
		this.date = date;
	}

	/**
	 * @param id
	 *            The id to set.
	 * @uml.property name="id"
	 */
	public void setId(Integer id) {
		this.id = id;
	}

	/**
	 * @param priceTotal
	 *            The priceTotal to set.
	 * @uml.property name="priceTotal"
	 */
	public void setPriceTotal(double d) {
		priceTotal = d;
	}

	public String orderItemstoString() {
		Iterator i = orderItems.iterator();
		String o = "";
		while (i.hasNext()) {
			OrderItem item = (OrderItem) i.next();
			o = o + item.getAmount() + " " + item.getProduct().getName() + "-";
		}
		return o;
	}

	public String toString() {
		String s = "[Order] id=" + id + "\n        priceTotal=" + priceTotal
				+ "\n        date=" + date + "\n";
		s = s + "Order items:\n";
		Iterator i = orderItems.iterator();
		String o = "";
		while (i.hasNext()) {
			OrderItem item = (OrderItem) i.next();
			o = o + "  " + item + "\n";
		}
		return s + o;
	}

	public String asString() {
		return  id +"  "+ date.toString()+ " "+priceTotal;
	}

	public Customer getCustomer() {
		return customer;
	}

	public void setCustomer(Customer customer) {
		this.customer = customer;
	}

}
