package app;

import generic.BaseObject;

import java.io.Serializable;

//import pojo.BaseObject;

/**
 * @author Pfister
 */
public class OrderItem extends BaseObject implements Serializable {

	private static final long serialVersionUID = -974019597084406167L;

	public OrderItem() {
		// empty default constructor
	}

	/**
	 * Creates valid OrderItem.
	 * 
	 * @param order
	 *            to which this OrderItem belongs
	 * @param p
	 *            from which this OrderItem is created
	 * @param amount
	 */
	public OrderItem(Order order, Product product, int amount) {
		this.order = order;
		this.product = product;
		this.amount = amount;
		product.setAmount(product.getAmount() - amount);
		this.price = product.getPrice() * amount;
	}

	/**
	 * 
	 * 
	 * @usage: orderItem.setProductAmount(product,Integer.parseInt((String)
	 *         request.getParameter("amount"),false));
	 */

	public void setProductAmount(Product product, int amount, boolean insertion) {
		String mesg = null;
		/*
		if (amount == 0) {  //sera rejet par la bd
			mesg = "amount+is+null";
			System.out.println(mesg);
		} else if (amount == 7) { //non rejet par la bd, mais rg
			mesg = "amount must be different of 7";
			System.out.println(mesg);
		} else 
		*/
		
		if (amount == 7) { // non rejet par la bd, mais rg
			mesg = "amount must be different of 7";
			System.out.println(mesg);
		}

		this.product = product;
		this.amount = amount;
		if (insertion) //  faire: prendre l'ancienne valeur, si diff corriger
						// le delta, sinon identique
			product.setAmount(product.getAmount() - amount);
		this.price = product.getPrice() * amount;

		if (mesg != null) {
			throw new RuntimeException(mesg); // will be rolled back after
												// rendering in view
		}

	}

	public String asString() {
		return amount + " " + product.getName() + " " + price;
	}

	private Integer id;

	/**
	 * @uml.property name="order"
	 * @uml.associationEnd inverse="orderItems:app.Order"
	 */
	private Order order;

	/**
	 * Getter of the property <tt>order</tt>
	 * 
	 * @return Returns the order.
	 * @uml.property name="order"
	 */
	public Order getOrder() {
		return order;
	}

	/**
	 * Setter of the property <tt>order</tt>
	 * 
	 * @param order
	 *            The order to set.
	 * @uml.property name="order"
	 */
	public void setOrder(Order order) {
		this.order = order;
	}

	private double price;
	private int amount;

	/**
	 * @return Returns the amount.
	 * @uml.property name="amount"
	 */
	public int getAmount() {
		return amount;
	}

	/**
	 * @return Returns the id.
	 * @uml.property name="id"
	 */
	public Integer getId() {
		return id;
	}

	/**
	 * @return Returns the price.
	 * @uml.property name="price"
	 */
	public double getPrice() {
		return price;
	}

	/**
	 * @param amount
	 *            The amount to set.
	 * @uml.property name="amount"
	 */
	public void setAmount(int i) {
		amount = i;
	}

	/**
	 * @param id
	 *            The id to set.
	 * @uml.property name="id"
	 */
	public void setId(Integer id) {
		this.id = id;
	}

	/**
	 * @param price
	 *            The price to set.
	 * @uml.property name="price"
	 */
	public void setPrice(double d) {
		price = d;
	}

	public String toString() {
		return "[OrderItem] id=" + id + "\n             amount=" + amount
				+ "\n             price=" + price + "\n             ("
				+ product + ")";
	}
	

	/**
	 * @uml.property name="product"
	 * @uml.associationEnd inverse="app.Product"
	 */
	private Product product;

	/**
	 * Getter of the property <tt>product</tt>
	 * 
	 * @return Returns the product.
	 * @uml.property name="product"
	 */
	public Product getProduct() {
		return product;
	}

	/**
	 * Setter of the property <tt>product</tt>
	 * 
	 * @param product
	 *            The product to set.
	 * @uml.property name="product"
	 */
	public void setProduct(Product product) {
		this.product = product;
	}

}
