package app.dao.jdbc;

import generic.BaseDao;
import generic.BaseObject;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import utils.DateUtils;

import app.Order;
import app.OrderItem;
import app.Product;

public class OrderItemDao implements BaseDao {

	private static Log log = LogFactory.getLog(OrderItemDao.class);

	private static MyStoreDatabase database;

	private static OrderItemDao instance;

	private OrderItemDao() {

	}

	public static synchronized OrderItemDao getInstance() {
		if (instance == null)
			try {
				database = MyStoreDatabase.getInstance();
				instance = new OrderItemDao();
			} catch (Exception e) {
				e.printStackTrace();
			}
		return instance;
	}

	public void save(OrderItem instance) {
		try {
			String request;
			OrderItem exists = findById(instance.getId());
			if (exists == null)
				request = "insert into order_items ( ORDER_ID, PRODUCT_ID, AMOUNT, PRICE) VALUES ("
						+ instance.getOrder().getId()
						+ ", "
						+ instance.getProduct().getId()
						+ ", "
						+ instance.getAmount()
						+ ", "
						+ instance.getPrice()
						+ ")";
			else {
				String orderIdClause = "";
				String productIdClause = "";
				if (instance.getOrder() != null)
					orderIdClause = " ORDER_ID=" + instance.getOrder().getId()
							+ ",";
				if (instance.getProduct() != null)
					productIdClause = " PRODUCT_ID="
							+ instance.getProduct().getId() + ",";

				request = "update order_items set " + orderIdClause
						+ productIdClause + " AMOUNT=" + instance.getAmount()
						+ ", PRICE=" + instance.getPrice() + " where ID="
						+ instance.getId();
				
				//request = "update order_items set ORDER_ID="+instance.getOrder().getId()+", PRODUCT_ID="+instance.getProduct().getId()+", AMOUNT="+instance.getAmount()+", PRICE="+instance.getPrice()+ " where ID="+ instance.getId();

			}
			log.debug(request);
			Connection c = database.getConnection();
			if (exists == null) {
				int generatedKey = database.insert(c, request);
				instance.setId(new Integer(generatedKey));
			} else
				database.update(c, request);
			c.close();
		} catch (Exception re) {
			log.error("attach failed", re);
			throw new RuntimeException(re);
		}
	}

	public void save(BaseObject instance) {
		save((OrderItem) instance);
	}

	public BaseObject findById(String id) {
		if (id == null)
			return null;
		try {
			return findById(new Integer(id));
		} catch (Exception re) {
			log.error("findById failed", re);
			return null;
		}
	}

	private OrderItem get(ResultSet rs) throws NumberFormatException,
			SQLException {
		OrderItem orderItem = new OrderItem();
		orderItem.setId(new Integer(rs.getString("ID")));
		orderItem.setAmount(rs.getInt("AMOUNT"));
		Product product = (Product) ProductDao.getInstance().findById(
				rs.getString("PRODUCT_ID"));
		orderItem.setProduct(product);
		orderItem.setPrice(rs.getDouble("PRICE"));
		return orderItem;
	}

	public OrderItem findById(Integer id) {
		if (id == null)
			return null;
		try {
			String query = "select * from order_items where ID="
					+ id.intValue();
			OrderItem orderItem = null;// = new OrderItem();
			Connection c = database.getConnection();
			ResultSet rs = database.executeQuery(c, query);
			if (rs.next())
				orderItem = get(rs);
			c.close();
			return orderItem;
		} catch (Exception re) {
			log.error("findById failed", re);
			return null;
		}
	}

	public List findByParent(Order order) {
		String filter = "where ORDER_ID = " + order.getId().toString();
		String query = "select * from order_items " + filter + " order by ID";
		log.debug(query);
		List result = new ArrayList();
		try {
			Connection c = database.getConnection();
			ResultSet rs = database.executeQuery(c, query);
			while (rs.next()) {
				OrderItem orderItem = orderItem = get(rs);
				orderItem.setOrder(order);

				result.add(orderItem);
			}
			c.close();
		} catch (Exception re) {
			log.error("find by ParentId failed", re);
			return null;
		}
		return result;
	}

	public List findByParentId(String id) {
		if (id != null && id.length() > 0) {
			Order order = (Order) OrderDao.getInstance().findById(id);
			return findByParent(order);
		} else
			return null;
	}

	public void delete(BaseObject instance) {
		deleteById(instance.getId().toString());
	}

	public void deleteById(String id) { // tester la rcupration des erreurs
		try {
			Connection c = database.getConnection();
			database.update(c, "delete from order_items where ID=" + id);
			c.close();
		} catch (Exception re) {
			log.error("delete failed", re);
			throw new RuntimeException(re);
		}
	}

}