import generic.BaseObject;
import generic.PersistenceException;

import java.util.Iterator;
import java.util.List;

import junit.framework.TestCase;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import utils.BeanUtils;

import app.Order;
import app.OrderItem;
import app.Product;
import app.dao.jdbc.OrderDao;
import app.dao.jdbc.OrderItemDao;
import app.dao.jdbc.ProductDao;

public class OrderItemTest extends TestCase{

	private static Log log = LogFactory.getLog(OrderItemTest.class);

	public static final int AMOUNT = 0;
	public static final int PRODUCT = 1;
	public static final int EXPECTED = 2;

	private static final String testData[][] = {
			{ "10", "26", "amount=10\n             price=28.8\n             ([Product] (26)Patati price=2.88 amount=100)" },
			{ "10", "24", "amount=10\n             price=293.76\n             ([Product] (24)Ding price=29.376 amount=100)" },
			{ "10", "23", "amount=10\n             price=1733.76\n             ([Product] (23)Bing price=173.376 amount=100)" } };

	private Order order;

	private OrderItem currentOrderItem;
	private int currentIndex;

	private static OrderItemDao orderItemDao = OrderItemDao.getInstance();

	protected void update() {
		currentOrderItem.setProductAmount(
				(Product) ProductDao.getInstance().findById(testData[currentIndex][PRODUCT]), 
				Integer.parseInt(testData[currentIndex][AMOUNT]), true);
	}
	
	
	protected void link() {
		currentOrderItem.setOrder(order);
		order.addOrderItem(currentOrderItem);
	}	

	protected List findByParentId(String id) throws PersistenceException {
		return orderItemDao.findByParentId(id);
	}

	protected BaseObject findById(String id) throws PersistenceException {
		return orderItemDao.findById(id);
	}

	protected void save() {
		orderItemDao.save(currentOrderItem);
	}

	protected void delete() {
		String msg = "delete " + currentOrderItem.getProduct().getName() + " ok";
		orderItemDao.delete(currentOrderItem);
		log.debug(msg);
	}

	protected OrderItem newObject() {
		return new OrderItem();
	}

	public String toString() {
		String result = "";
		try {
			List orderItems = findByParentId(order.getId().toString());
			Iterator ordItemIterator = orderItems.iterator();
			while (ordItemIterator.hasNext()) {
				OrderItem orderItem = (OrderItem) ordItemIterator.next();
				result += orderItem;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return result;
	}

	public void create(int index) throws PersistenceException {
		currentOrderItem = newObject();
		currentIndex = index;
		update();
		link();
		save();
		String result = toString();
		assertTrue(result.indexOf(testData[currentIndex][EXPECTED]) >= 0);
		log.debug("create " + currentOrderItem + " ok");
	}
	
	
	public OrderItem find(Integer id) throws PersistenceException {
		OrderItem result = (OrderItem) findById(id.toString());
		assertNotNull(result);
		log.debug("find " + result + " ok");
		return result;
	}	

	public OrderItem[] getOrderItems() {
		List result = BeanUtils.toList(order.getOrderItems());
		return (OrderItem[]) result.toArray(new OrderItem[result.size()]);
	}

	public void createAllWithParent(Order[] o) throws PersistenceException {
		for (int i = 0; i < o.length; i++) {
			createWithParent(o[i]);
		}
	}

	public void deleteAll(Order[] o) throws  PersistenceException {
		for (int i = 0; i < o.length; i++) {
			deleteWithParent(o[i]);
		}
	}

	public void createWithParent(Order o) throws  PersistenceException {
		order = o;
		for (int i = 0; i < testData.length; i++) {
			create(i);
			find(currentOrderItem.getId());
		}
	}

	public void deleteWithParent(Order order) throws  PersistenceException {
		this.order = order;
		OrderItem[] orderitems = getOrderItems();
		for (int i = 0; i < orderitems.length; i++) {
			currentOrderItem = orderitems[i];
			delete();
		}
	}

	public void testCreateDelete() {
		Order order = (Order) OrderDao.getInstance().findById("18"); // on sait qu'il existe dans la base de donnes
		try {
			createWithParent(order);
			log.info(order);
			deleteWithParent(order);	
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	public void testFindUpdate() {
			try {
			currentOrderItem = find(new Integer(43));
			currentOrderItem.setAmount(42);
			save();			
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}	

}
