import generic.BaseObject;
import generic.PersistenceException;

import java.util.Iterator;
import java.util.List;

import junit.framework.TestCase;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import utils.BeanUtils;
import utils.DateUtils;
import app.Customer;
import app.Order;
import app.dao.jdbc.CustomerDao;
import app.dao.jdbc.OrderDao;

public class OrderTest extends TestCase{

	private static Log log = LogFactory.getLog(OrderTest.class);
	public static final int DATE = 0;
	public static final int PRICE = 1;
	public static final int EXPECTED = 2;

	private static final String testData[][] = { 
		    { "04/04/2006", "100.99", "priceTotal=100.99\n        date=Tue Apr 04 00:00:00 CEST 2006\n" }, // pb CEST a																																// voir
			{ "31/12/2006", "101.95", "priceTotal=101.95\n        date=Sun Dec 31 00:00:00 CET 2006\n" },
			{ "12/12/2006", "100.5", "priceTotal=100.5\n        date=Tue Dec 12 00:00:00 CET 2006\n" } };


	private Order currentOrder;
	private int currentIndex;

	private OrderItemTest orderItemTest = new OrderItemTest();

	private Customer customer;

	private static OrderDao orderDao = OrderDao.getInstance();

	protected void update() {
		currentOrder.setDate(DateUtils.stringDMY2Date(testData[currentIndex][DATE]));
		currentOrder.setPriceTotal(Double.parseDouble(testData[currentIndex][PRICE]));
	}
	
	protected void link() {
		currentOrder.setCustomer(customer);
		customer.addOrder(currentOrder);
	}	

	protected List findByParentId(String id) throws PersistenceException {
		return orderDao.findByParentId(id);
	}

	protected BaseObject findById(String id) throws PersistenceException {
		return orderDao.findById(id);
	}

	protected void save() {
		orderDao.save(currentOrder);
	}

	protected void delete() {
		String msg = currentOrder.getDate().toString();
		orderDao.delete(currentOrder);
		log.debug("delete order "+msg+ "ok");
	}

	protected Order newObject() {
		return new Order();
	}

	public String toString() {
		String result = "";
		try {
			List orders = findByParentId(customer.getId().toString());
			Iterator ordIterator = orders.iterator();
			while (ordIterator.hasNext()) {
				Order order = (Order) ordIterator.next();
				result += order;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return result;
	}

	public void deleteAll() throws PersistenceException {
		Order[] orders = getOrders();
		for (int i = 0; i < orders.length; i++){
			currentOrder = orders[i];
			delete();
		}	
	}

	public void create(int ind) throws PersistenceException {
		currentIndex = ind;
		currentOrder = newObject();
		update();
		link();
		save();
		String result = toString();
		assertTrue(result.indexOf(testData[currentIndex][EXPECTED]) >= 0);
		log.debug("create " + currentOrder + " ok");
	}
	
	
	public Order find(Integer id) throws PersistenceException {
		Order o  = (Order) findById(id.toString());
		assertNotNull(o);
		log.debug("find order " + id + " ok");
		return o;
	}	
	
	public void createWithParent(Customer customer) throws PersistenceException {
		this.customer = customer;
		for (int i = 0; i < testData.length; i++){ 
			create(i);
			find(currentOrder.getId());
		}
	}

	public Order getFirstOrder() {
		return getOrders()[0];
	}

	public Order[] getOrders() {
		List result = BeanUtils.toList(customer.getOrders());
		return (Order[]) result.toArray(new Order[result.size()]);	
	}

	public void createOrderItems() throws PersistenceException {
		Order[] orders = getOrders();
		for (int i = 0; i < orders.length; i++)
			orderItemTest.createWithParent(orders[i]);
	}

	public void deleteOrderItems() throws PersistenceException {
		Order[] orders = getOrders();
		for (int i = 0; i < orders.length; i++)
		  orderItemTest.deleteWithParent(orders[i]);
	}

	public void testCreateDelete()  {
		try {
			Customer customer = (Customer) CustomerDao.getInstance().findById("1"); // on sait qu'il existe dans la base de donnes
			createWithParent(customer);
			createOrderItems();
			log.info(customer);
			deleteOrderItems();
			deleteAll();			
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	public void testFindUpdate()  {
		try {
			currentOrder = find(new Integer(18));
			//Customer customer = (Customer) CustomerDao.getInstance().findById(new Integer(rs.getString("CUSTOMER_ID")));
			//order.setCustomer(customer);
			currentOrder.setPriceTotal(12.999);
			save();		
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	




}
