package servlet;

import generic.BaseDao;
import generic.BaseObject;
import generic.PersistenceException;

import java.io.IOException;
import java.util.List;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;



public abstract class BaseServlet extends HttpServlet {

	private static Log log = LogFactory.getLog(BaseServlet.class);

	protected BaseDao dao;
	protected BaseDao parentDao;
	protected String beanName;
	protected String listName;
	protected String forward;
	protected List beanList;

	protected String cmd;
	
	public void init(ServletConfig arg0) throws ServletException {
		super.init(arg0);
	}	
	
	protected abstract void setParentBean(BaseObject parent);
			
	
	protected List findByParentId(String id) throws PersistenceException {
		return dao.findByParentId(id);
	}
	
	protected BaseObject findParent(String id) throws PersistenceException {
		if (parentDao != null)
		  return parentDao.findById(id);
		else
		  return null;
	}	

	protected BaseObject findById(String id) throws PersistenceException {
		return dao.findById(id);
	}

	protected BaseObject find(Integer id) throws PersistenceException {
		log.debug("find " + id);
		BaseObject o = findById(id.toString());
		return o;
	}

	protected void save() {
		log.debug("save ");
		update();
		link();
		dao.save(getBean());
	}

	protected void delete() {
		log.debug("delete " + getBean().getId());
		unlink();
		dao.delete(getBean());
		newBean();
	}
	
	protected void beforeDispatch(HttpServletRequest request,
			HttpServletResponse response){
		//surcharger
	}
	
	protected void setReferences(){
        //surcharger
	}

	protected synchronized void doGet(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		try {
			clear();
			cmd = request.getParameter("cmd");
			String id = request.getParameter("id");
			String parentId = request.getParameter("parentid");
			BaseObject parent = findParent(parentId);
			setParentBean(parent);

			getParameters(request);

			if ("ajouter".equals(cmd)) {
				id = ""; // pour forcer la cration d'un nouvel objet
			}

			if (!"null".equals(id) && !"".equals(id) && id!=null)
			    setBean(find(new Integer(id)));
			else
				newBean();
			
			setReferences(); //pour les associations n - 1

			if ("modifier".equals(cmd) || "ajouter".equals(cmd))
				save();
			else if ("supprimer".equals(cmd))
				delete();

			beanList = findByParentId(parentId); // liste de tous les ..
			request.setAttribute("parent_"+beanName, parent);
			request.setAttribute(beanName, getBean());
			request.setAttribute(listName, beanList);
			beforeDispatch(request,response);
			RequestDispatcher dispatcher = request
					.getRequestDispatcher(forward);
			dispatcher.forward(request, response);

		} catch (PersistenceException e) {
			response.getWriter().println("error " + e.getMessage());
			e.printStackTrace();
		}
	}

	protected void doPost(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		doGet(request, response);
	}
	
	protected abstract void update();
	
	protected abstract void link();
	
	protected abstract void unlink();

	protected abstract void getParameters(HttpServletRequest request);

	protected abstract void clear();

	protected abstract void newBean();
	
	protected abstract BaseObject getBean();
	
	protected abstract void setBean(BaseObject bean);
		

}
